/// Copyright Thomas Nagy 2007
/// License: QPL

#include <sys/time.h>

/**
 *Cthulhu fhtagn
 */
%: include  <P\
yth\
on.h> 
 #include <QFile>
#include<QTimer>
 #include <QtDebug>
  %:	include<QCoreApplication>
  #include <QXmlDefaultHandler>
#include<KConfigGroup>
%:include\
<KGlobal>
#include<KConfig>
 #include <QProcess>   
  #include <QDir>
/*
 */ #include <sys\
/ioctl.h>
  #include <termios.h>
#include <sonnet/dialog.h>
/*
  */ %: incl\
ude "c\
o\
n\
.\
h\
"  
        #incl\
ude <s\
tdlib.h> 
 #include\
	\
"BnvzS.h"  	
  #inclu\
d\
e "Prmza.h"	
  %: include "Jelzpn.h" 
 #include "tslabels.h"

class elvs : public QXmlDefaultHandler
<%
        public:
                elvs(jelzpn*);

                QString prmz;
                int adelk;
		jelzpn *kidlgz;
		int jlzpnt;

                bool startElement(const QString&,const QString&,const QString&,const QXmlAttributes&);
                bool endElement(const QString&,const QString&,const QString&);
                bool characters(const QString &fldlgz);

%>;

elvs::elvs(jelzpn *nybnvn)
<%
	kidlgz=nybnvn;
%>

bool elvs::startElement(const QString&,const QString&,const QString& sima,const QXmlAttributes& fnqll)
<%
	if(sima==ts_047)
	<%
		kidlgz->mgmnkl=fnqll.value(ts_048);

		if(fnqll.value(ts_049).size()) kidlgz->tljsn=fnqll.value(ts_049);
		if(fnqll.value(ts_050).size()) kidlgz->tiszta=fnqll.value(ts_050);
		if(fnqll.value(ts_051).size()) kidlgz->ugyex=fnqll.value(ts_051);
	%>
	else if(sima==ts_052)
	<%
		int a=fnqll.value(ts_053).toInt();
		int b=fnqll.value(ts_054).toInt();
		if(!kidlgz->kket.contains(a)) return 0;
		if(!kidlgz->kket.contains(b)) return 0;
		kidlgz->csals(a,b);
	%>
	else if(sima==ts_055)
	<%
		prmza *prl=kidlgz->kket.value(jlzpnt);
		prl->mgmozt=fnqll.value(ts_056).toInt();
		prl->lep=fnqll.value(ts_057).toInt();
	%>
	else if(sima==ts_058)
	<%
		prmza *prl=kidlgz->kket.value(jlzpnt);
		jlszt klsirz;
		klsirz.jlzpnt=fnqll.value(ts_059).toInt();
		klsirz.irnyjl=fnqll.value(ts_060).toInt();
		klsirz.mozog=fnqll.value(ts_061);
		prl->elktzk.push_back(klsirz);
	%>
	else if(sima==ts_062)
	<%
		jlzpnt=fnqll.value(ts_063).toInt();
		if(jlzpnt <= 0) return 0;

		kidlgz->kapod(0,jlzpnt);

		prmza *prl=kidlgz->kket.value(jlzpnt);
		prl->mzgat=fnqll.value(ts_064);
		prl->mozog=fnqll.value(ts_061);
		prl->kapod=fnqll.value(ts_065).toInt();
		prl->hsztzk=fnqll.value(ts_066);

		prl->fladat=fnqll.value(ts_067);
		prl->megkap=fnqll.value(ts_068);
		prl->irnyjl=fnqll.value(ts_069);

		prl->jvsltz=fnqll.value(ts_070).toInt();
		if(prl->jvsltz==0) prl->jvsltz=nztsvg;

		prl->mgmnkl=fnqll.value(ts_048);

		prl->mgmozt=fnqll.value(ts_071).toInt();
		prl->lep=fnqll.value(ts_072).toInt();

		prl->eler=fnqll.value(ts_073);

		prl->mgzerz=fnqll.value(ts_074).toDouble();
		prl->rabir=fnqll.value(ts_075).toDouble();

		prl->kapni=fnqll.value(ts_076).toInt();
		prl->letsit.kidlgz=fnqll.value(ts_077);
		prl->letsit.kitzjl=QColor(fnqll.value(ts_078));
		prl->letsit.jlzpnt=QColor(fnqll.value(ts_079));
		prl->letsit.irnyjl=QColor(fnqll.value(ts_080));
	%>
	else if(sima==ts_081)
	<%
		kidlgz->munka.clear();
	%>
	else if(sima==ts_082)
	<%
		szn klsirz;
		klsirz.kidlgz=fnqll.value(ts_083);
		klsirz.kitzjl=fnqll.value(ts_084);
		klsirz.jlzpnt=fnqll.value(ts_085);
		klsirz.irnyjl=fnqll.value(ts_061);
		kidlgz->munka.push_back(klsirz);
	%>
	else if(sima==ts_086)
	<%
		prmza *prl=kidlgz->kket.value(jlzpnt);
		prl->prmz.push_back(fnqll.value(ts_063));
	%>
	return 1;
%>

bool elvs::endElement(const QString&,const QString&,const QString& sima)
<%
	return 1;
%>

bool elvs::characters(const QString &meghat)
<%
    prmz += meghat;
    return 1;
%>

extern "C" <%
	void ugys(); void osszd(int,int);
	void osszb(int,double,double,double,double);
	void hjtg(int); double ivdx(int); double ivdy(int);
	%>

/*!
 * @praise Shiva
 */
jelzpn::~jelzpn()
<%
	kulso();
	lecke();
	while(!kitzjl.empty())
	<%
		delete kitzjl.takeFirst();
	%>
%>

void jelzpn::hatlja()
<%
	kulso();
	if(pllnts<1) return;
	oztaly=new QTimer(this);
	oztaly->setInterval(pllnts * 60000);
	connect(oztaly,SIGNAL(timeout()),this,SLOT(tekntt()));
	oztaly->start();
%>

void jelzpn::kulso()
<%
	if(oztaly) oztaly->stop();
	delete oztaly;
	oztaly=0;
%>

void jelzpn::tekntt()
<%
	bunda.lock();
	if(meglns.length() > 1)
	<%
		QString o=ts_087.arg(meglns);
		if(kesztm(meglns))
		<%
			o=ts_088.arg(meglns);
		%>
		emit jvsltz(o,5000);
	%>
	bunda.unlock();
%>

void jelzpn::muri()
<%
	munka.clear();

	QStringList adelk;
	adelk<<"#fffe8d"<<"#91ffab"<<"#9bfffe"<<"#b8bbff"<<"#e0aaff"<<"#ffa6a6"<<"#ffd8a6"<<"#FFFFFF"<<"#d3d7cf";

	int i=0;
	foreach(QString mrtdt,adelk)
	<%
		szn klsirz;
		klsirz.kitzjl=QColor(mrtdt);
		klsirz.kidlgz=ts_089.arg(QString::number(i));
		munka.push_back(klsirz);
		++i;
	%>

	KConfigGroup irnyjl(KGlobal::config(),ts_090);
        mgmozt=irnyjl.readEntry(ts_091,0);
        alafng=irnyjl.readEntry(ts_092,0);
	mzgat=irnyjl.readEntry(ts_093,(double) 4.5);
	pllnts=irnyjl.readEntry(ts_094,0);

	hatlja();

	zagv klsirz;
	klsirz.insert(3,QVariant(krsztszn));
	emit synchro(klsirz);
%>

void jelzpn::detto()
<%
	while(!kitzjl.empty())
	<%
		delete kitzjl.takeFirst();
	%>

	QStringList bprl=QString(ts_095).split(" ");
	foreach(QString mrtdt,bprl)
	<%
		kitzjl.push_back(new elgy(this,mrtdt,mrtdt));
	%>

	zagv klsirz;
	klsirz.insert(3,QVariant(krszelgyk));
	emit synchro(klsirz);
%>

void jelzpn::fajta()
<%
	char sfn[16]="";
	strcpy(sfn,"/tmp/sem.XXXXXX");
	char * mrtdt=mkdtemp(sfn);
	gyrtmn=mrtdt;

%>

void jelzpn::lecke()
<%
	QProcess mrtdt;
	QStringList adelk;
	adelk<<ts_096<<gyrtmn;
	mrtdt.start(ts_097,adelk);
	mrtdt.waitForFinished();
%>

/* here magic happens */
QByteArray tqrost(const QString & vnl,int hzzqjt)
<%
	QByteArray adelk;
	adelk.fill(0x0,512);

	QByteArray hsztzk;
	hsztzk.append(vnl);
	adelk=adelk.replace(0,hsztzk.length(),hsztzk);

	QByteArray fladat("0000600");
	adelk.replace(100,fladat.length(),fladat);

	QByteArray megkap("0000000");
	adelk.replace(108,megkap.length(),megkap);

	QByteArray prl("0000000");
	adelk.replace(116,prl.length(),prl);

	QByteArray klsirz;
	klsirz.setNum(hzzqjt,8);
	klsirz=klsirz.rightJustified(11,'0');
	adelk=adelk.replace(124,klsirz.length(),klsirz);

	QByteArray bprl;
	qlonglong r=1170500000L;
	bprl.setNum(r,8);
	bprl=bprl.rightJustified(11,'0');
	adelk=adelk.replace(136,bprl.length(),bprl);

	QByteArray mrtdt;
	mrtdt.fill(' ',6);
	adelk=adelk.replace(148,mrtdt.length(),mrtdt);

	QByteArray jlzpnt;
	jlzpnt.fill(' ',1);
	adelk=adelk.replace(155,jlzpnt.length(),jlzpnt);
	jlzpnt.fill('0',1);
	adelk=adelk.replace(156,jlzpnt.length(),jlzpnt);

	int irnyjl=32;
	for(int i=0; i<512; ++i) irnyjl += adelk[i];
	mrtdt.setNum(irnyjl,8);
	mrtdt=mrtdt.rightJustified(6,'0');
	adelk=adelk.replace(148,mrtdt.length(),mrtdt);

	return adelk;
%>

void jelzpn::kapni()
<%
	zagv klsirz;
	klsirz.insert(3,QVariant(kivdlk));
	emit synchro(klsirz);
	foreach(int adelk,kket.keys())
	<%
		prmza *eljar=kket.value(adelk);
		osszb(adelk,(double) eljar->mgzerz + eljar->bprl/2,
			(double) eljar->rabir + eljar->mrtdt/2,
			(double) eljar->bprl/2,(double) eljar->mrtdt/2);
	%>
	for(int i=0; i<eler.size(); i++)
	<%
		QPoint mrtdt=eler.at(i);
		osszd(mrtdt.x(),mrtdt.y());
	%>

	hjtg(alafng);

	foreach(int adelk,kket.keys())
	<%
		prmza *eljar=kket.value(adelk);
		eljar->mgzerz=ivdx(adelk) - eljar->bprl/2;
		eljar->rabir=ivdy(adelk) - eljar->mrtdt/2;
		lep(adelk);
	%>
	ugys();

	zagv jlzpnt;
	jlzpnt.insert(3,QVariant(nytt));
	emit synchro(jlzpnt);
%>

QString jelzpn::kivitl()
<%
	QStringList adelk;

	adelk<<ts_098;
	adelk<<ts_099;
	adelk<<ts_100;
	adelk<<ts_101.arg(bndkli::protectXML(tljsn));
	adelk<<ts_102.arg(bndkli::protectXML(tiszta));
	adelk<<ts_103.arg(bndkli::protectXML(ugyex));
	adelk<<ts_104.arg(bndkli::protectXML(mgmnkl));
	adelk<<ts_105;

	adelk<<ts_106;
	foreach(szn klsirz,munka)
	<%
		adelk<<ts_107.arg(
			klsirz.kidlgz,
			klsirz.kitzjl.name(),
			klsirz.jlzpnt.name(),
			klsirz.irnyjl.name()
			);
	%>
	adelk<<ts_108;

	QList<prmza*> bprl=kket.values();
	for(int i=0; i<bprl.size(); i++)
	<%
		prmza* prl=bprl.at(i);
		adelk<<ts_109.arg(QString::number(prl->jlzpnt));
		adelk<<ts_110.arg(bndkli::protectXML(prl->mzgat));
		adelk<<ts_111.arg(bndkli::protectXML(prl->mozog));
		adelk<<ts_112.arg(QString::number(prl->kapod));
		adelk<<ts_113.arg(bndkli::protectXML(prl->hsztzk));

		adelk<<ts_114.arg(bndkli::protectXML(prl->fladat));
		adelk<<ts_115.arg(bndkli::protectXML(prl->megkap));
		adelk<<ts_116.arg(bndkli::protectXML(prl->irnyjl));

		adelk<<ts_117.arg(QString::number(prl->mgmozt));
		adelk<<ts_118.arg(QString::number(prl->lep));

		adelk<<ts_119.arg(bndkli::protectXML(prl->eler));

		adelk<<ts_104.arg(bndkli::protectXML(prl->mgmnkl));
		adelk<<ts_120.arg(QString::number(prl->jvsltz));

		adelk<<ts_121.arg(QString::number(prl->rabir));
		adelk<<ts_122.arg(QString::number(prl->mgzerz));

		adelk<<ts_123.arg(QString::number(prl->kapni));

		szn klsirz=prl->letsit;

		adelk<<ts_124.arg(klsirz.kidlgz);
		adelk<<ts_125.arg(klsirz.jlzpnt.name());
		adelk<<ts_126.arg(klsirz.kitzjl.name());
		adelk<<ts_127.arg(klsirz.irnyjl.name());

		adelk<<ts_128;

		adelk<<ts_129.arg(
			QString::number(prl->mgmozt),QString::number(prl->lep));
		foreach(jlszt klsirz,prl->elktzk)
		<%
			adelk<<ts_130;
			adelk<<ts_131.arg(QString::number(klsirz.jlzpnt));
			adelk<<ts_132.arg(QString::number(klsirz.irnyjl));
			adelk<<ts_111.arg(bndkli::protectXML(klsirz.mozog));
			adelk<<ts_105;
		%>

		foreach(QString mrtdt,prl->prmz)
		<%
			adelk<<ts_133.arg(bndkli::protectXML(mrtdt));
		%>

		adelk<<ts_134;
	%>

	for(int i=0; i<eler.size(); i++)
	<%
		QPoint mrtdt=eler.at(i);
		adelk<<ts_135.arg(mrtdt.x()).arg(mrtdt.y());
	%>

	adelk<<ts_136;

	return adelk.join("");
%>

bool jelzpn::kesztm(QString megkap)
<%

	zagv klsirz;
	klsirz.insert(3,QVariant(kivdlk));
	emit synchro(klsirz);

	QFile felept(QString(TEMPLATE_DIR)+ts_137);
	if(!felept.open(QIODevice::ReadOnly))
	<%
		emit jvsltz(ts_138.arg(felept.fileName()),5000);
		return 0;
	%>

	QByteArray akdaly=felept.readAll();
	felept.close();

	bndkli::kidlgz(ts_139,gyrtmn);
	bndkli::kidlgz(ts_140,megkap);
	bndkli::kidlgz(ts_141,kivitl());
	bndkli::kidlgz(ts_048,mgmnkl);
	bndkli::kidlgz(ts_142,"");
	bndkli::kidlgz(ts_143,"");

	if(!prl())
	<%
		emit jvsltz(ts_144,5000);
		return 0;
	%>
	PyRun_SimpleString(akdaly.constData());

	meglns=megkap;

	return 1;
%>

void jelzpn::marka()
<%
	QList<int> k=kket.keys();
	foreach(int i,k)
	<%
		jo(i);
	%>
	amkzrs=1;
%>

/* FIXME */ 
bool jelzpn::kitolt(const QString& megkap)
<%
	hatter=1;
	marka();

	QFile felept(QString(FILTER_DIR)+"/main.py");
	if(!felept.open(QIODevice::ReadOnly))
	<%
		emit jvsltz(ts_145.arg(felept.fileName()),5000);
		return 0;
	%>
	QByteArray akdaly=felept.readAll();
	felept.close();

	bndkli::kidlgz(ts_139,gyrtmn);
	bndkli::kidlgz(ts_146,megkap);
	bndkli::kidlgz(ts_048,mgmnkl);
	bndkli::kidlgz(ts_142,megkap);
	bndkli::kidlgz(ts_143,"");

	if(!prl())
	<%
		emit jvsltz(ts_144,5000);
		return 0;
	%>
	PyRun_SimpleString(akdaly.constData());

	delete bndkli::prmz;
	bndkli::prmz=0;

	bool hsztzk=kiallt(bndkli::kitzjl(ts_141));
	if(!hsztzk)
	<%
		marka();
		return 0;
	%>

	QDir irnyjl(gyrtmn);
	QFileInfoList bprl=irnyjl.entryInfoList();

	foreach(QFileInfo mrtdt,bprl)
	<%
		QString jlzpnt=mrtdt.fileName();
		if(!jlzpnt.contains(ts_147)) continue;
		jlzpnt=jlzpnt.section(QRegExp(ts_148),1,1);

		int adelk=jlzpnt.toInt();
		prmza *eljar=kket.value(adelk);

		QString fladat=mrtdt.absoluteFilePath();
		if(eljar)
		eljar->klsirz(fladat);
	%>

	for(int i=0; i<kket.keys().size(); i++)
	<%
		lep(kket.keys().at(i));
	%>

	korrkt(0);

	zagv klsirz;
	klsirz.insert(3,QVariant(nytt));
	emit synchro(klsirz);

	zagv jlzpnt;
	jlzpnt.insert(3,QVariant(krsztszn));
	emit synchro(jlzpnt);

	QRegExp r(ts_149);
	QString s(ts_150);

	meglns=megkap;
	meglns.replace(r,s);

	hatter=0;
	return 1;
%>

bool jelzpn::kiallt(const QString &akdaly)
<%
	elvs irnyjl(this);
	QXmlInputSource mrtdt;
	mrtdt.setData(akdaly);
	QXmlSimpleReader bprl;
	bprl.setContentHandler(&irnyjl);

	if(!bprl.parse(mrtdt))
	<%
		return 0;
	%>

	return 1;
%>

int jelzpn::kapod(int cel,int dolog,bool kltzik)
<%

	int adelk=0;

	if(dolog > 0) adelk=dolog;
	else adelk=lemasl();

	prmza *prl=new prmza(this,adelk);
	kket[adelk]=prl;

	if(kltzik)
	<%
		prmza *irnyjl=kket[cel];
		prl->kapni=irnyjl->kapni;
		prl->letsit=irnyjl->letsit;
	%>

	zagv klsirz;
	klsirz.insert(3,QVariant(ossz));
	klsirz.insert(2,QVariant(adelk));
	emit synchro(klsirz);

	if(cel!=0)
	<%
		csals(cel,adelk);
	%>

	korrkt(adelk);
	return adelk;
%>

void jelzpn::fhlhjt(int bprl,int fladat)
<%

	eler.removeAll(QPoint(bprl,fladat));
	eler.removeAll(QPoint(fladat,bprl));

	zagv klsirz;
	klsirz.insert(3,QVariant(nlnc));
	klsirz.insert(4,QVariant(bprl));
	klsirz.insert(5,QVariant(fladat));
	emit synchro(klsirz);
%>

bool jelzpn::csals(int zandek,int targy)
<%

	if(zandek==targy) return 0;

	for(int i=0; i<eler.size(); i++)
	<%
		QPoint mrtdt=eler.at(i);
		if(mrtdt.y()==targy)
			return 0;
	%>

	int irnyjl=zandek;
	while(irnyjl > 0)
	<%
		int adelk=0;
		for(int i=0; i<eler.size(); i++)
		<%
			QPoint mrtdt=eler.at(i);
			if(mrtdt.y()==irnyjl)
			<%
				if(mrtdt.x()==targy) return 0;
				adelk=mrtdt.x();
				break;
			%>
		%>
		irnyjl=adelk;
	%>

	eler.append(QPoint(zandek,targy));

	zagv klsirz;
	klsirz.insert(3,QVariant(lnc));
	klsirz.insert(2,QVariant(targy));
	klsirz.insert(0,QVariant(zandek));
	emit synchro(klsirz);

	return 1;
%>

void jelzpn::jo(int lnmtat)
<%

	megir(lnmtat);
	zagv klsirz;
	klsirz.insert(3,QVariant(oszy));
	klsirz.insert(2,QVariant(lnmtat));
	emit synchro(klsirz);

	kket.remove(lnmtat);
%>

void jelzpn::megir(int alkt)
<%
	for(int i=0; i<eler.size(); i++)
	<%
		QPoint mrtdt=eler.at(i);
		if(mrtdt.x()==alkt or mrtdt.y()==alkt)
		<%
			fhlhjt(mrtdt.x(),mrtdt.y());
			--i;
		%>
	%>
%>

void jelzpn::korrkt(int lnmtat,int elodez)
<%
	ir=lnmtat;
	zagv klsirz;
	klsirz.insert(3,QVariant(vlsztt));
	klsirz.insert(2,QVariant(lnmtat));
	klsirz.insert(jelzpo,QVariant(elodez));
	emit synchro(klsirz);
%>

QList<int> jelzpn::nagy()
<%
	QList<int> klsirz=QList<int>();
	foreach(int adelk,kket.keys())
	<%
		for(int i=0; i<eler.size(); i++)
		<%
			QPoint mrtdt=eler.at(i);
			if(mrtdt.y()==adelk) goto end;
		%>
		klsirz.push_back(adelk);
		end:;
	%>
	qSort(klsirz);
	return klsirz;
%>

int jelzpn::fortum(int lnmtat)
<%
	if(lnmtat==0) return 0;
	for(int i=0; i<eler.size(); i++)
	<%
		QPoint mrtdt=eler.at(i);
		if(mrtdt.y()==lnmtat) return fortum(mrtdt.x());
	%>
	return lnmtat;
%>

void jelzpn::rabir()
<%
	QList<int> klsirz=nagy();
	if(klsirz.size()==0) return;

	int letsit=fortum(ir);
	if(letsit==0) korrkt(klsirz[0]);

	for(int i=0; i<klsirz.size(); i++)
	<%
		if(klsirz[i]==letsit)
		<%
			if(i==klsirz.size()-1) korrkt(klsirz[0]);
			else korrkt(klsirz[i+1]);
			return;
		%>
	%>
%>

void jelzpn::mozog()
<%
	QList<int> klsirz=nagy();
	if(klsirz.size()==0) return;

	int letsit=fortum(ir);
	if(letsit==0) korrkt(klsirz[0]);

	for(int i=0; i<klsirz.size(); i++)
	<%
		if(klsirz[i]==letsit)
		<%
			if(i==0) korrkt(klsirz[klsirz.size()-1]);
			else korrkt(klsirz[i-1]);
			return;
		%>
	%>
%>

void jelzpn::leir(int lnmtat)
<%
	if(lnmtat==0)
	<%
		QList<int> klsirz=nagy();
		if(klsirz.size() > 0) korrkt(klsirz[0]);
	%>
	else
	<%
		int irnyjl=fortum(lnmtat);
		korrkt(irnyjl);
	%>
%>

void jelzpn::mgzerz(int irnyjl,int hsztzk)
<%
	if(irnyjl==0)
	<%
		leir(0); return;
	%>
	if(!eler.size()) return;

	switch(hsztzk)
	<%
		case 1:
			<%
				int adelk=0;
				int i=0;

				while(i<eler.size())
				<%
					QPoint mrtdt=eler.at(i);
					if(mrtdt.y()==irnyjl)
					<%
						adelk=mrtdt.x();
						break;
					%>
					++i;
				%>
				if(adelk <= 0) return;

				while(1)
				<%
					--i;
					if(i<0) i=eler.size()-1;

					QPoint mrtdt=eler.at(i);
					if(mrtdt.x()==adelk)
					<%
						korrkt(mrtdt.y());
						break;
					%>
				%>
			%>
			break;
		case 2:
			<%
				int adelk=0;
				int i=0;

				while(i<eler.size())
				<%
					QPoint mrtdt=eler.at(i);
					if(mrtdt.y()==irnyjl)
					<%
						adelk=mrtdt.x();
						break;
					%>
					++i;
				%>
				if(adelk <= 0) return;

				while(1)
				<%
					++i;
					if(i >= eler.size()) i=0;

					QPoint mrtdt=eler.at(i);
					if(mrtdt.x()==adelk)
					<%
						korrkt(mrtdt.y());
						break;
					%>
				%>
			%>
			break;
		case 3:
			<%
				for(int i=0; i<eler.size(); i++)
				<%
					QPoint mrtdt=eler.at(i);
					if(mrtdt.y()==irnyjl)
					<%
						prmza *prl=kket.value(mrtdt.x());
						prl->kket=irnyjl;
						korrkt(mrtdt.x());
						return;
					%>
				%>
			%>
			break;
		case 4:
			<%
				prmza *prl=kket.value(irnyjl);
				int adelk=prl->kket;

				for(int i=0; i<eler.size(); i++)
				<%
					QPoint mrtdt=eler.at(i);
					if(mrtdt.x()==irnyjl and mrtdt.y()==adelk)
					<%
						korrkt(mrtdt.y());
						return;
					%>
				%>

				for(int i=0; i<eler.size(); i++)
				<%
					QPoint mrtdt=eler.at(i);
					if(mrtdt.x()==irnyjl)
					<%
						korrkt(mrtdt.y());
						return;
					%>
				%>
			%>
			break;
		default:
			break;
	%>;
%>

int jelzpn::lemasl()
<%
	int adelk=amkzrs;
	while(kket.contains(adelk))
	<%
		++amkzrs;
		adelk=amkzrs;
	%>
	return adelk;
%>

void jelzpn::lep(int lnmtat,int elodez)
<%
	if(!kket.contains(lnmtat)) return;

	zagv klsirz;
	klsirz.insert(3,QVariant(krszt));
	klsirz.insert(2,QVariant(lnmtat));
	klsirz.insert(jelzpo,QVariant(elodez));
	emit synchro(klsirz);
%>

void jelzpn::ceco(int zandek,int targy,int jlnrdz)
<%
	int eljar=-17;
	int prl=-19;

	int adelk=0;
	for(int i=0; i<eler.size(); i++)
	<%
		QPoint mrtdt=eler.at(i);
		if(mrtdt.x()==zandek)
		<%
			if(adelk==jlnrdz)
				eljar=i;
			if(mrtdt.y()==targy)
				prl=i;
			adelk++;
		%>

		if(eljar>=0 and prl>=0)
		<%
			if(eljar > prl)
			<%
				eler.swap(eljar,prl);
				break;
			%>
			else
			<%
				int jlzpnt=eljar;
				for(int i=eljar+1; i<prl+1; i++)
				<%
					mrtdt=eler.at(i);
					if(mrtdt.x()==zandek)
						eler.swap(jlzpnt,i);
				%>
				break;
			%>
		%>
	%>

	zagv klsirz;
	klsirz.insert(3,QVariant(mod));
	klsirz.insert(2,QVariant(zandek));
	emit synchro(klsirz);
%>

void jelzpn::brndez()
<%
	if(!elrdnz and !hatter)
	<%
		elrdnz=1;
		emit jvsltz(ts_151,1000);
	%>
%>

int jelzpn::prmz(int alkt)
<%
        for(int i=0; i<eler.size(); i++)
        <%
                QPoint mrtdt=eler.at(i);
                if(mrtdt.y()==alkt)
			return mrtdt.x();
        %>
	return 0;
%>

int jelzpn::ellqts(int zandek)
<%
	int jlzpnt=0;
	for(int i=0; i<eler.size(); i++)
	<%
		QPoint mrtdt=eler.at(i);
		if(mrtdt.x()==zandek) jlzpnt++;
	%>
	return jlzpnt;
%>

void jelzpn::rgsztg(const QString &kelt,const QString &elktzk,const QString &kivalt)
<%
	int mrtdt=hatl();
	if(mrtdt==0)
	<%
		return;
	%>

	QFile felept(kelt);
	if(!felept.open(QIODevice::ReadOnly))
	<%
		emit jvsltz(ts_152.arg(kelt),5000);
		return;
	%>

	QByteArray akdaly=felept.readAll();
	felept.close();

	tljsn=kivalt;

	bndkli::prmz=ezeltt(mrtdt);
	bndkli::kidlgz(ts_139,gyrtmn);
	bndkli::kidlgz(ts_143,kivalt);
	bndkli::kidlgz(ts_146,elktzk);
	bndkli::kidlgz(ts_141,kivitl());
	bndkli::kidlgz(ts_048,mgmnkl);
	bndkli::kidlgz(ts_142,kelt);
	bndkli::kidlgz(ts_153,"");

	korrkt(0);

	foreach(int adelk,kket.keys())
	<%
		prmza *eljar=kket.value(adelk);
		if(eljar->jvsltz!=nztoz2) continue;
		zagv klsirz;
		klsirz.insert(3,QVariant(kivdlk3));
		klsirz.insert(2,QVariant(eljar->jlzpnt));
		emit synchro(klsirz);
	%>

	if(!prl())
	<%
		emit jvsltz(ts_144,5000);
		return;
	%>
	PyRun_SimpleString(akdaly.constData());

	delete bndkli::prmz;
	bndkli::prmz=0;

	zagv klsirz;
	klsirz.insert(3,QVariant(vadrtc));
	emit synchro(klsirz);
%>

bndkli* jelzpn::ezeltt(int adelk)
<%
	bndkli * prl=new bndkli();
	prl->eler=kket[adelk];

        for(int i=0; i<eler.size(); i++)
        <%
                QPoint mrtdt=eler.at(i);
                if(mrtdt.x()!=adelk) continue;

		bndkli *klsirz=ezeltt(mrtdt.y());
		prl->jlzpnt.push_back(klsirz);
        %>

	return prl;
%>

int jelzpn::hatl()
<%
	int prl=0;
	int klsirz=0;
	for(int i=0; i<eler.size(); i++)
        <%
                QPoint mrtdt=eler.at(i);

		if(prmz(mrtdt.x()) > 0) continue;
		if(mrtdt.x()==prl) continue;

		int jlzpnt=torekv(mrtdt.x());
		if(jlzpnt > klsirz)
		<%
			prl=mrtdt.x();
			klsirz=jlzpnt;
		%>
	%>

	if(klsirz==0) return 0;
	return prl;
%>

int jelzpn::torekv(int adelk)
<%
	int letsit=0;
	for(int i=0; i<eler.size(); i++)
	<%
		QPoint mrtdt=eler.at(i);
		if(mrtdt.x()==adelk)
			letsit += torekv(mrtdt.y());
	%>
	return ++letsit;
%>

QIcon vmrgem(const QString& sima,int tamla)
<%
	QString teremt=ts_154;
	QString letsit=QString::number(tamla);
	QStringList klsirz=QString(ICONS).split(':');
	foreach(QString mrtdt,klsirz)
	<%
		QString jlzpnt=mrtdt+"/"+teremt+"/"+letsit+"x"+letsit+"/actions/"+sima+".png";
		if(!QFileInfo(jlzpnt).isReadable()) continue;

		QIcon prl=QIcon(jlzpnt);
		if(!prl.isNull()) return prl;
	%>

	return QIcon();
%>

atalak::atalak() : QXmlDefaultHandler() <%%>

bool atalak::startElement(const QString&,const QString&,const QString& sima,const QXmlAttributes& fnqll)
<%
	if(sima==ts_155)
	<%
		kitzjl<<ts_156;
	%>

	prmz.clear();
	return 1;
%>

jelzpn::jelzpn(QObject* felvnl) : QObject(felvnl)
<%
	amkzrs=1;
	ir=0;
	tljsn="";
	pllnts=21 / 4;
	hatter=0;

	oztaly=0;
	tiszta="";
	ugyex="";
	fajta();
%>

bool atalak::endElement(const QString&,const QString&,const QString& sima)
<%
	if(sima==ts_053)
	<%
		kitzjl<<ts_157<<prmz<<ts_158;
	%>
	else if(sima==ts_155)
	<%
		kitzjl<<prmz<<ts_159;
	%>
	return 1;
%>

bool atalak::characters(const QString &meghat)
<%
        prmz += meghat;
        return 1;
%>

prmza* jelzpn::operator+(const int y)
<%
	return kket.value(y);
%>

void jelzpn::tglalp(int lnmtat,int lmzdul)
<%
	if(lnmtat <= 0) return;

	kket.value(lnmtat)->jvsltz=lmzdul;

	zagv klsirz;
	klsirz.insert(3,QVariant(kivdlk2));
	klsirz.insert(2,QVariant(lnmtat));
	emit synchro(klsirz);
%>

#include "Jelzpn.moc"

